"""Constants for the BSB-Lan integration."""

DOMAIN = "bsblan"

DATA_BSBLAN_CLIENT = "bsblan_client"
DATA_BSBLAN_TIMER = "bsblan_timer"
DATA_BSBLAN_UPDATED = "bsblan_updated"

ATTR_IDENTIFIERS = "identifiers"
ATTR_MODEL = "model"
ATTR_MANUFACTURER = "manufacturer"

ATTR_TARGET_TEMPERATURE = "target_temperature"
ATTR_INSIDE_TEMPERATURE = "inside_temperature"
ATTR_OUTSIDE_TEMPERATURE = "outside_temperature"

ATTR_STATE_ON = "on"
ATTR_STATE_OFF = "off"

CONF_DEVICE_IDENT = "device_identification"
CONF_CONTROLLER_FAM = "controller_family"
CONF_CONTROLLER_VARI = "controller_variant"

SENSOR_TYPE_TEMPERATURE = "temperature"

CONF_PASSKEY = "passkey"
