"""Constants for the Local File Camera component."""
DOMAIN = "local_file"
SERVICE_UPDATE_FILE_PATH = "update_file_path"
CONF_FILE_PATH = "file_path"
DATA_LOCAL_FILE = "local_file_cameras"
DEFAULT_NAME = "Local File"
