"""Constants of the Xiaomi Aqara component."""

DOMAIN = "xiaomi_aqara"

GATEWAYS_KEY = "gateways"
LISTENER_KEY = "listener"

ZEROCONF_GATEWAY = "lumi-gateway"

CONF_INTERFACE = "interface"
CONF_PROTOCOL = "protocol"
CONF_KEY = "key"
CONF_SID = "sid"

DEFAULT_DISCOVERY_RETRY = 5
